﻿//!CompilerOption:AddRef:SharpSvn.dll
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using SVNUpdater.Helpers;
using SharpSvn;
using Styx.CommonBot;
using Styx.Plugins;
using Styx.WoWInternals.WoWObjects;
using SVNUpdater;
namespace SVNUpdater.Gui
{
    public partial class frmConfiguration : Form
    {
        /*
         * These Params should be used as public ones by the Plugin, not private ones from the form
         */ 
        private IPluginLogger _logger;
        private IList<Thread> _threads;
        // This variable should be filled by current count of settings / SVN Downloads, we need it to see if the threads are finished
        int toProcess = 3;
        ManualResetEvent resetEvent = new ManualResetEvent(false);
        public frmConfiguration()
        {
            if(_logger==null) _logger = new PluginLogger();
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            _logger.Write("Settings saved, rebuilding behaviors...");
            base.OnClosing(e);
        }
        //this should be outsourced to helper class
        public Thread StartSVNDownload(string LocalPath, string remotePath, string targetPath)
        {
            var t = new Thread(() => DownloadSVNRepository(LocalPath, remotePath, targetPath));
            return t;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (RuntimePolicyHelper.LegacyV2RuntimeEnabledSuccessfully)
            {
                _threads = new List<Thread>();
                //here we should iterate through settings
                StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Routines"), "http://subversion.assembla.com/svn/purerotation/trunk/PureRotation","PureRotation");
                StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Routines"), "http://king-wow.googlecode.com/svn/trunk","King WoW");
                StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Routines"), "https://subversion.assembla.com/svn/singular_v3/trunk/Singular","Singular");
                StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Routines"), "https://subversion.assembla.com/svn/team-random/trunk/release/routines/Fury%20Unleashed/dotnet_4_5/", "Fury Unleashed");
				/*Example for a Plugin*/
				//StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Plugins"), "yourPluginSVN","ThePluginFolderName");
				/*Example for a BotBase*/
				//StartInNewThread(Path.Combine(Styx.Common.Utilities.AssemblyDirectory, "Bots"), "yourBotSVN","TheBotFolderName");	
				/*Anything else, maybe not related to Honorbuddy? no Problem*/
				//StartInNewThread("FolderSomeWhereOnYourComp", "yourSomethingSVN","TheSomeThingSubdirectory");					
                resetEvent.WaitOne();
                _logger.Write("Everything updated");
            }
        }
        //this should be outsourced to helper class
        void StartInNewThread(string LocalPath, string remotePath, string targetPath)
        {
            Thread thread = new Thread(
                delegate() { 
                    _logger.Write(string.Format("Starting Thread with ID {0}",Thread.CurrentThread.ManagedThreadId));
                    DownloadSVNRepository(LocalPath, remotePath, targetPath);
                    if (Interlocked.Decrement(ref toProcess) == 0)
                        resetEvent.Set();
                });
            thread.IsBackground = true;
            thread.Name = string.Format("MyThread{0}", "MAD");
            _threads.Add(thread);
            thread.Start();    
        }
        //this should be outsourced to helper class
        void ThreadEntry(string LocalPath,string remotePath,string targetPath)
        {
            DownloadSVNRepository(LocalPath, remotePath, targetPath);
        }
        //this should be outsourced to helper class
        private void DownloadSVNRepository(string localPath, string remotePath, string targetPath)
        {
            try
            {
                _logger.Write(string.Format("Updating to {0} from {1}", localPath, remotePath));
                SvnUpdateResult result;
                long revision;
                SvnCheckOutArgs args = new SvnCheckOutArgs();
                string path = localPath;//We will built this dynamically";
                string[] s = remotePath.Split('/');
                int i = s.Length;
                string temppath = "";

                if (targetPath.Length > 0)
                {
                    temppath = targetPath;
                    path = Path.Combine(localPath, temppath);
                    _logger.Write(string.Format("Path built to {0} by using path from targetPath", path));
                }
                else
                {
                    temppath = s[i - 1].Length == 0 || s[i - 1] == "trunk" ? s[i - 2] : s[i - 1];
                    _logger.Write(string.Format("{0} / {1} - {2}", temppath, s[i - 1], s[i - 2]));
                    path = Path.Combine(localPath, temppath);
                    _logger.Write(string.Format("Path built to {0} by extracting path from remotePath", path));
                }
                if (temppath.Length == 0)
                {

                    path = Path.Combine(localPath, remotePath.GetHashCode().ToString());
                    _logger.Write(string.Format("Path built to {0} by hashing remotePath", path));
                }
                if (long.TryParse(remotePath, out revision))
                {
                    args.Revision = new SvnRevision(revision);
                    _logger.Write("Revision built");
                }
                else _logger.Write("Invalid Revision number, defaulting to HEAD");
                using (SvnClient client = new SvnClient())
                {
                    try
                    {
                        SvnUriTarget target = new SvnUriTarget(remotePath);
                        if (client.CheckOut(target, path, args, out result))
                        _logger.Write("Successfully checked out revision " + result.Revision + ".");
                    }
                    catch (SvnException se)
                    {
                        _logger.Write(se.Message);
                    }
                    catch (UriFormatException ufe)
                    {
                        _logger.Write(ufe.Message);
                    }
                }
            }
            catch(Exception ex) 
            {
                _logger.Write(ex.ToString());
            }
        }
    }
}
